#! /usr/bin/env python
#
# Simple tests for debug.py, based on following:
#     https://stackoverflow.com/questions/16039463/how-to-access-the-py-test-capsys-from-inside-a-test
#
# Notes:
# - For tests that capture standard error, see
#       https://docs.pytest.org/en/6.2.x/capture.html
# - This uses capsys fixture mentioned in above link.
#................................................................................
# TODO:
# - make sure trace_fmt traps all exceptiona
#   debug.trace_fmt(1, "fu={fu}", fuu=1)
#                           ^^    ^^^
#

"""Tests for debug module"""

# Standard packages
## OLD: import unittest

# Installed packages
import pytest

# Local packages
from mezcla import debug

class TestDebug:
    """Class for test case definitions"""
    stdout_text = None
    stderr_text = None
    expected_stdout_trace = None
    expected_stderr_trace = None

    def setup_simple_trace(self):
        """Common setup for simple tracing"""
        debug.trace(4, f"setup_common(): self={self}")
        self.stdout_text = "hello"
        self.stderr_text = "world"
        self.expected_stdout_trace = self.stdout_text + "\n"
        self.expected_stderr_trace = self.stderr_text + "\n"

    def test_visible_simple_trace(self, capsys):
        """Make sure level-1 trace outputs to stderr"""
        debug.trace(4, f"test_visible_simple_trace({capsys})")
        self.setup_simple_trace()
        if not __debug__:
            self.expected_stderr_trace = ""
        pre_captured = capsys.readouterr()
        save_trace_level = debug.get_level()
        debug.set_level(4)
        print(self.stdout_text)
        debug.trace(3, self.stderr_text)
        debug.set_level(save_trace_level)
        captured = capsys.readouterr()
        assert(captured.out == self.expected_stdout_trace)
        assert(captured.err == self.expected_stderr_trace)
        debug.trace_expr(6, pre_captured, captured)

    def test_hidden_simple_trace(self, capsys):
        """Make sure level-N+1 trace doesn't output to stderr"""
        debug.trace(4, f"test_hidden_simple_trace({capsys})")
        self.setup_simple_trace()
        ## TEST
        ## capsys.stop_capturing()
        ## capsys.start_capturing()
        pre_captured = capsys.readouterr()
        self.expected_stderr_trace = ""
        save_trace_level = debug.get_level()
        debug.set_level(0)
        print(self.stdout_text)
        debug.trace(1, self.stderr_text)
        captured = capsys.readouterr()
        debug.set_level(save_trace_level)
        assert(captured.out == self.expected_stdout_trace)
        assert(captured.err == self.expected_stderr_trace)
        debug.trace_expr(6, pre_captured, captured)

    def test_debug_val(self):
        """Make sure debug.val only returns value when at specified level"""
        debug.trace(4, f"test_debug_val(): self={self}")
        save_trace_level = debug.get_level()
        test_value = 22
        debug.set_level(5)
        level5_value = debug.val(5, test_value)
        debug.set_level(0)
        level0_value = debug.val(1, test_value)
        debug.set_level(save_trace_level)
        assert(level5_value == test_value)
        assert(level0_value is None)
        
    def test_debug_code(self):
        """Make sure debug code not executed at all"""
        debug.trace(4, f"test_debug_value(): self={self}")
        ## TODO: debug.assertion(debug_level, debug.code(debug_level, lambda: (8 / 0 != 0.0)))
        save_trace_level = debug.get_level()
        count = 0
        def increment():
            """Increase counter"""
            nonlocal count
            count += 1
        debug.set_level(4)
        debug.code(4, lambda: increment)
        debug.set_level(save_trace_level)
        assert(count == 0)
        
#------------------------------------------------------------------------

if __name__ == '__main__':
    debug.trace_current_context()
    pytest.main([__file__])



#! /usr/bin/env python
#
# Test(s) for ../my_regex.py
#
# Notes:
# - For debugging the tested script, the ALLOW_SUBCOMMAND_TRACING environment
#   option shows tracing output normally suppressed by  unittest_wrapper.py.
# - This can be run as follows:
#   $ PYTHONPATH="$(realpath .)/..):$PYTHONPATH" python tests/test_my_regex.py
#

"""Tests for my_regex module"""

# Standard packages
import re
import unittest

# Installed packages
## TODO: import pytest

# Local packages
from mezcla.unittest_wrapper import TestWrapper
from mezcla import debug

# Note: Two references are used for the module to be tested:
#    THE_MODULE:	    global module object
#    TestIt.script_module   string name
import mezcla.my_regex as THE_MODULE

class TestIt(TestWrapper):
    """Class for testcase definition"""
    script_module = TestWrapper.get_testing_module_name(__file__)
    my_re = THE_MODULE.my_re

    def test_simple_regex(self):
        """"Test regex search with capturing"""
        debug.trace(4, "test_simple_regex()")
        if not self.my_re.search(r"(\w+)\W+(\w+)", ">scrap ~!@\n#$ yard<",
                                 re.MULTILINE):
            self.fail("simple regex search failed")
        self.assertEqual(self.my_re.group(1), "scrap")
        self.assertEqual(self.my_re.group(2), "yard")
        return

#------------------------------------------------------------------------

if __name__ == '__main__':
    debug.trace_current_context()
    unittest.main()




#! /usr/bin/env python
#
# Test(s) for glue_helpers.py. This can be run as follows:
# $ PYTHONPATH="." tests/test_glue_helpers.py
#
# TODO:
# - Add support for write_lines & read_lines.
# - Add support for other commonly used functions.
#

"""Tests for glue_helpers module"""

# Installed packages
import os
import unittest

# Local packages
from mezcla.unittest_wrapper import TestWrapper
import mezcla.glue_helpers as gh


class TestIt(TestWrapper):
    """Class for testcase definition"""
    script_module = TestWrapper.derive_tested_module_name(__file__)

    def test_extract_matches(self):
        """Tests for extract_matches(pattern, lines)"""
        ## OLD
        ## self.assertEqual(gh.extract_matches("Mr. (\S+)", ["Mr. Smith", "Mr. Jones", "Mr.X"]), ["Smith", "Jones"])
        ## self.assertNotEqual(gh.extract_matches("\t\S+", ["abc\tdef", "123\t456"]), ["def", "456"])
        self.assertEqual(gh.extract_matches(r"Mr. (\S+)", ["Mr. Smith", "Mr. Jones", "Mr.X"]), ["Smith", "Jones"])
        self.assertNotEqual(gh.extract_matches(r"\t\S+", ["abc\tdef", "123\t456"]), ["def", "456"])
        return

    def test_basename(self):
        """Tests for basename(path, extension)"""
        self.assertEqual(gh.basename("fubar.py", ".py"), "fubar")
        self.assertNotEqual(gh.basename("fubar.py", ""), "fubar")
        return

    def test_resolve_path(self):
        """Tests for resolve_path(filename)"""
        script = "glue_helpers.py"
        test_script = "test_glue_helpers.py"
        # The main script should resolve to parent directory but this one to test dir
        self.assertNotEqual(gh.resolve_path(script), 
                            os.path.join(os.path.dirname(__file__), test_script))
        self.assertEqual(gh.resolve_path(test_script), 
                         os.path.join(os.path.dirname(__file__), test_script))
        return


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
#
# Test(s) for ../text_utils.py
#
# Notes:
# - For debugging the tested script, the ALLOW_SUBCOMMAND_TRACING environment
#   option shows tracing output normally suppressed by  unittest_wrapper.py.
# - This can be run as follows:
#   $ PYTHONPATH="$(realpath .)/..):$PYTHONPATH" python tests/test_text_utils.py
#

"""Tests for text_utils module"""

# Standard packages
import re
import unittest

# Installed packages
## TODO: import pytest

# Local packages
from mezcla.unittest_wrapper import TestWrapper
from mezcla import debug
from mezcla import glue_helpers as gh
from mezcla import system

# Note: Two references are used for the module to be tested:
#    THE_MODULE:	    global module object
#    TestIt.script_module   string name
import mezcla.text_utils as THE_MODULE

HTML_FILENAME = "simple-window-dimensions.html"

EXPECTED_TEXT = """
   Simple window dimensions

   Simple window dimensions

      Legend:
        Screen dimensions:    ???
        Browser dimensions:   ???

   JavaScript is required
"""
#
# NOTE: Whitespace and punctuation gets normalized
# TODO: restore bullet points (e.g., "* Screen dimensions")

MS_WORD_FILENAME = "spanish-accents.docx"

MS_WORD_TEXT = "Tío Tomás\t\t\t\tUncle Tom\n\n¡Buenos días!\t\t\t\tGood morning\n\nçãêâôöèäàÃëÇÂîòïÔìðÊÅåùÀŠý\t\tcaeaooeaaAeCAioiOioEAauASy"


def normalize_test_text(text):
    """Trim excess whitespace and convert punctuation to <PUNCT>"""
    # EX: normalize_test_text("   h  e y?! ") => "h e y<PUNCT>"
    result = text.strip()
    result = re.sub(r"\s+", " ", result)
    result = re.sub(r"[^\w\s]+", "<PUNCT>", result)
    debug.trace(4, f"normalize_test_text({text}) => {result}")
    return result
    

class TestIt(TestWrapper):
    """Class for testcase definition"""
    script_module = TestWrapper.get_testing_module_name(__file__)

    def test_html_to_text(self):
        """Ensure html_to_text works as expected"""
        # TODO: move into test_html_utils.py
        debug.trace(4, "test_html_to_text()")
        html_path = gh.resolve_path(HTML_FILENAME)
        html = system.read_file(html_path)
        text = THE_MODULE.html_to_text(html)
        self.assertEqual(normalize_test_text(text),
                         normalize_test_text(EXPECTED_TEXT))
        return

    def test_document_to_text(self):
        """Ensure document_to_text works as expected"""
        debug.trace(4, "test_document_to_text()")
        doc_path = gh.resolve_path(MS_WORD_FILENAME)
        text = THE_MODULE.document_to_text(doc_path)
        self.assertEqual(normalize_test_text(text),
                         normalize_test_text(MS_WORD_TEXT))
        ## OLD: self.assertEqual(1, 2)
    
#------------------------------------------------------------------------

if __name__ == '__main__':
    debug.trace_current_context()
    unittest.main()



#! /usr/bin/env python
#
# Test(s) for ../html_utils.py
#
# Notes:
# - Fill out TODO's below. Use numbered tests to order (e.g., test_1_usage).
# - TODO: If any of the setup/cleanup methods defined, make sure to invoke base
#   (see examples below for setUp and tearDown).
# - This can be run as follows:
#   $ PYTHONPATH=".:$PYTHONPATH" python tests/test_html_utils.py
#

"""Tests for html_utils module"""

# Standard packages
import re
import unittest

# Installed packages
## TODO: import pytest

# Local packages
from mezcla.unittest_wrapper import TestWrapper
from mezcla import debug
from mezcla import system
from mezcla import glue_helpers as gh

# Note: Two references are used for the module to be tested:
#    THE_MODULE:	    global module object
#    TestIt.script_module   string name
## TODO: template => new name
import mezcla.html_utils as THE_MODULE
#
# Note: sanity test for customization (TODO: remove if desired)
if not re.search(__file__, r"\btemplate.py$"):
    debug.assertion("mezcla.template" not in str(THE_MODULE))

# 
TEST_SELENIUM = system.getenv_bool("TEST_SELENIUM", False,
                                   "Include tests requiring selenium")

class TestIt(TestWrapper):
    """Class for testcase definition"""
    script_module = TestWrapper.get_testing_module_name(__file__)
    # TODO: use_temp_base_dir = True            # treat TEMP_BASE as directory
    # note: temp_file defined by parent (along with script_module, temp_base, and test_num)


    def test_get_url_parameter_value(self):
        """Ensure get_url_parameter_value works as expected"""
        debug.trace(4, "test_get_url_parameter_value()")
        save_user_parameters = THE_MODULE.user_parameters
        THE_MODULE.user_parameters = {}
        self.assertEqual(THE_MODULE.get_url_parameter_value("fubar", None), None)
        self.assertEqual(THE_MODULE.get_url_parameter_value("fubar", None, {"fubar": "fu"}), "fu")
        THE_MODULE.user_parameters = {"fubar": "bar"}
        self.assertEqual(THE_MODULE.get_url_parameter_value("fubar", None), "bar")
        self.assertEqual(THE_MODULE.get_url_parameter_value("fubar", None, {"fubar": "fu"}), "fu")
        THE_MODULE.user_parameters = save_user_parameters
        return

    def test_get_inner_text(self):
        """Verify that JavaScript fills in window dimensions
        Note: requires selenium"""
        debug.trace(4, "test_get_inner_text()")
        if not TEST_SELENIUM:
            debug.trace(4, "Ignoring test_get_inner_text as selenium required")
            return
        html_filename = "simple-window-dimensions.html"
        html_path = gh.resolve_path(html_filename)
        url = ("file:" + system.absolute_path(html_path))
        # TODO: use direct API call to return unrendered text
        unrendered_text = gh.run(f"lynx -dump {url}")
        debug.trace_expr(5, unrendered_text)
        self.assertTrue(re.search(r"Browser dimensions: \?", unrendered_text))
        rendered_text = THE_MODULE.get_inner_text(url)
        debug.trace_expr(5, rendered_text)
        self.assertTrue(re.search(r"Browser dimensions: \d+x\d+", rendered_text))
    
    def test_get_inner_html(self):
        """Verify that JavaScript fills in window dimensions
        Note: requires selenium"""
        debug.trace(4, "test_get_inner_html()")
        if not TEST_SELENIUM:
            debug.trace(4, "Ignoring test_get_inner_html as selenium required")
            return
        html_filename = "simple-window-dimensions.html"
        html_path = gh.resolve_path(html_filename)
        url = ("file:" + system.absolute_path(html_path))
        # TODO: use direct API call to return unrendered text
        unrendered_html = gh.run(f"lynx -source {url}")
        debug.trace_expr(5, unrendered_html)
        self.assertTrue(re.search(r"<li>Browser dimensions:\s*<span.*>\?\?\?</span></li>",
                                  unrendered_html))
        rendered_html = THE_MODULE.get_inner_html(url)
        debug.trace_expr(5, rendered_html)
        self.assertTrue(re.search(r"<li>Browser dimensions:\s*<span.*>\d+x\d+</span></li>",
                                  rendered_html))
    
#------------------------------------------------------------------------

if __name__ == '__main__':
    debug.trace_current_context()
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Test(s) for tpo_common.py
#
# Notes:
# - This can be run as follows:
#   $ PYTHONPATH="." python tests/test_tpo_common.py
# TODO:
# - Address commonly used debugging functions (e.g., debug_print) by redirecting output (via remapping sys.stderr to a file) and then checking file contents.
# - add tests for normalize_unicode, ensure_unicode and other problematic functions
#

"""Tests for tpo_common module"""

# Standard packages
import os
import unittest

# Local packages
from mezcla import tpo_common as tpo

FUBAR = 101	# sample global for test_format
FOOBAR = 12     # likewise
JOSE = "José"   # UTF-8 encoded string

class TestIt(unittest.TestCase):
    """Class for testcase definition"""

    def test_format(self):
        """Ensure format resolves from local and global namespace, and that local takes precedence"""
        fubar = 202
        self.assertEqual(tpo.format("{F} vs. {f}", F=FUBAR, f=fubar), 
                         ("%s vs. %s" % (FUBAR, fubar)))
        # pylint: disable=redefined-outer-name
        FOOBAR = 21
        self.assertEqual(tpo.format("{FOO}", FOO=FOOBAR), 
                         str(FOOBAR))
        # TODO: self.assertEqual("Hey Jos\xc3\xa9", tpo.format("Hey {j}", j=JOSE))
        return

    def test_get_current_function_name(self):
        """Test(s) for get_current_function_name()"""
        self.assertEqual(tpo.get_current_function_name(), "test_get_current_function_name")
        return

    def test_getenv_functions(self):
        """Ensure that various getenv_xyz functions work as expected"""
        self.assertEqual(tpo.getenv_integer("REALLY FUBAR", 123), 123)
        self.assertEqual(tpo.getenv_number("REALLY FUBAR", 123), 123.0)
        self.assertFalse(isinstance(tpo.getenv_boolean("REALLY FUBAR?", None), 
                                    bool))
        self.assertTrue(isinstance(tpo.getenv_boolean("REALLY FUBAR?", False), 
                                   bool))
        self.assertFalse(isinstance(tpo.getenv_text("REALLY FUBAR?", False), 
                                    bool))
        os.environ["FUBAR"] = "1"
        self.assertEqual(tpo.getenv_text("FUBAR"), "1")
        return

    def test_unicode_functions(self):
        """Esnure that normalize_unicode, encode_unicode, etc. work as expected"""
        UTF8_BOM = "\xEF\xBB\xBF"
        self.assertEqual(tpo.ensure_unicode("ASCII"), u"ASCII")
        self.assertEqual(tpo.normalize_unicode("ASCII"), "ASCII")
        ## TODO: self.assertEqual(tpo.ensure_unicode(UTF8_BOM), u'\ufeff')
        self.assertEqual(tpo.normalize_unicode(UTF8_BOM), UTF8_BOM)
        self.assertEqual(u"Jos\xe9", tpo.ensure_unicode(JOSE))
        ## TODO: self.assertEqual("Jos\xc3\xa9", tpo.normalize_unicode(JOSE))
        return
        
    def test_difference(self):
        """Ensures set difference works as expected"""
        self.assertEqual(tpo.difference([1, 2, 3], [2]), 
                         [1, 3])
        self.assertEqual(tpo.difference([1, 1, 2, 2], [1]), 
                         [2])
        return

if __name__ == '__main__':
    unittest.main()
